import json

from common.utils import fernet
from django.core.management.base import BaseCommand
from django.core.serializers.python import Deserializer as PythonDeserializer
from pathlib import Path
from uuid import UUID
from django.db import DEFAULT_DB_ALIAS
from django.db import transaction

map_names = {
    'account': 'account.customer',
    'customer': 'account.customer',
    'source': 'ticketsource.ticketsource',
    'contact': 'csat.contact',
    'company': 'csat.company',
    'logo': 'configuration.logo',
    'cw_partner': 'sso.cwpartner',
    'user': 'account.user',
    'domain': 'configuration.domain',
    'session': 'session'
}

class AlwaysNone:
    def __getitem__(self, key):
        return None

ids_map = dict(
    session = AlwaysNone()
)


def deserialize(objects, **options):
    yield from PythonDeserializer(objects, **options)



class Command(BaseCommand):
    help = 'Import customer data to database.'

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
        self.customer_id = None
        self.format= 'json'
        self.path = Path('./out')

    # def add_arguments(self, parser):
    #     parser.add_argument('--dump', '-d', action='store', type=pathlib.Path,
    #                         help='Dump file')



    def _create_objects(self, dump, fid_fields=None):
        if fid_fields:
            for obj in dump:
                if isinstance(fid_fields, list) or isinstance(fid_fields, tuple):
                    for field_name in fid_fields:
                        if field_name in obj['fields'] and obj['fields'][field_name]:
                            obj['fields'][field_name] = ids_map[map_names[field_name]][obj['fields'][field_name]]
                else:
                    for field_name, model_name in fid_fields.items():
                        if field_name in obj['fields'] and obj['fields'][field_name]:
                            if isinstance(obj['fields'][field_name], str):
                                obj['fields'][field_name] = ids_map[model_name][UUID(obj['fields'][field_name])]
                            else:
                                obj['fields'][field_name] = ids_map[model_name][obj['fields'][field_name]]

        objects = deserialize(
            dump,
            using=DEFAULT_DB_ALIAS,
            ignorenonexistent=True,
            handle_forward_references=False,
        )
        for obj in objects:
            old = obj.object.pk
            obj.object.pk = None
            obj.save()
            map = ids_map.get(str(obj.object._meta), {})
            map[old] = obj.object.pk
            ids_map[str(obj.object._meta)] = map

    def _load_dump(self, name):
        dump_path = self.path / name
        if dump_path.exists() and dump_path.stat().st_size != 0:
            return json.load(open(self.path / name, encoding='utf8'))
        return []

    def create_objects(self, name, fid_fields=None):
        dump = self._load_dump(name)
        return self._create_objects(dump, fid_fields)

    def create_tss(self):
        dump = self._load_dump('tss.json')
        for tss in dump:
            if tss['fields'].get('credentials', {}).get('cs_credentials', {}).get('privateKey'):
                cs_private_key = fernet.encrypt(tss['fields']['credentials']['cs_credentials']['privateKey']).decode()
                tss['fields']['credentials']['privateKey'] = cs_private_key
            if tss['fields'].get('credentials', {}).get('private_key'):
                private_key = fernet.encrypt(tss['fields']['credentials']['private_key']).decode()
                tss['fields']['credentials']['private_key'] = private_key
        return self._create_objects(dump, ('customer',))

    def create_nps(self):
        self.create_objects('nps_contacts.json', ('customer',))
        dump = self._load_dump('nps_campaigns_companies.json')
        for obj in dump:
            if obj['model'] == 'nps.campaign':
                resources = obj['fields']['resources']
                if resources.get('fixed_contact_list'):
                    if resources['tss_query'].get('excluded'):
                        new_excluded = []
                        for id in resources['tss_query'].get('excluded'):
                            new_excluded.append(str(ids_map['nps.contact'][int(id)]))
                        resources['tss_query'] = new_excluded
                    filter_ids = resources['tss_query'].get('applied_filters', {}).get('ids')
                    if filter_ids:
                        new_filter_ids = []
                        for id in filter_ids:
                            new_filter_ids.append(str(ids_map['nps.contact'][int(id)]))
                        resources['tss_query']['applied_filters']['ids'] = new_filter_ids
                    obj['fields']['resources'] = resources

        self._create_objects(dump, ('logo', 'custom_domain', 'customer'))
        self.create_objects('nps_survey_csv_import.json', {'company': 'nps.company',
                                                           'campaign': 'nps.campaign',
                                                           'customer': 'account.customer'})

        dump = self._load_dump('nps_responses_imported_contacts.json')
        for obj in dump:
            if obj['model'] == 'nps.importedcontacts' and obj['fields']['inner']:
                ids = json.loads(obj['fields']['ids'].encode().decode('unicode_escape'))
                new_ids = []
                for id in ids:
                    new_ids.append(str(ids_map['nps.contact'][int(id)]))
                obj['fields']['ids'] = new_ids

            if obj['model'] == 'nps.response':
                if obj['fields']['from_id'] == str(obj['fields']['contact']):
                    obj['fields']['from_id'] = str(ids_map['nps.contact'][obj['fields']['contact']])



        self._create_objects(dump, {'survey': 'nps.survey', 'contact': 'nps.contact',
                                    'customer': 'account.customer'})

    def create_configurations(self):
        dump = self._load_dump('configuration.json')
        for obj in dump:
            if obj['model'] == 'nps.unsubscribedcontacts':
                if obj['fields']['inner_ids']:
                    ids = json.loads(obj['fields']['inner_ids'].encode().decode('unicode_escape'))
                    new_ids = []
                    for id in ids:
                        new_ids.append(str(ids_map['nps.contact'][int(id)]))
                    obj['fields']['inner_ids'] = new_ids
        self._create_objects(dump, ('account', 'customer'))

    def create_projects(self):

        self.create_objects('projects_phases.json', {'survey': 'projects.survey'})
        self.create_objects('projects_triggers.json', {'survey': 'projects.survey',
                                                       'phase': 'projects.phase'})

        dump = self._load_dump('projects_rest.json')
        global_categories = json.load(open(self.path / 'global_categories.json'))
        for obj in dump:
            if obj['model'] == 'projects.question' and obj['fields']['category'] not in global_categories:
                obj['fields']['category'] = ids_map['projects.category'][obj['fields']['category']]

        self._create_objects(dump, {'contact': 'projects.contact',
                                    'survey': 'projects.survey',
                                    'trigger': 'projects.trigger'})

        self.create_objects('projects_responses.json', {'question': 'projects.question',
                                                        'deployment': 'projects.deployment'})

    def create_automations(self):
        dump = self._load_dump('automations.json')
        for obj in dump:
            if obj['model'] == 'automation.automationrule':
                if obj['fields']['type'] == 'nps':
                    trigger = obj['fields']['trigger']
                    if trigger.get('campaigns'):
                        new_camaigns = []
                        for camp in trigger.get('campaigns'):
                            new_camaigns.append(ids_map['nps.campaign'][camp])
                        obj['fields']['trigger']['campaigns'] = new_camaigns
                if obj['fields']['type'] == 'project':
                    trigger = obj['fields']['trigger']
                    if trigger.get('surveys'):
                        new_surveys = []
                        print(ids_map['projects.survey'])
                        for sur in trigger.get('surveys'):
                            print(sur)
                            new_surveys.append(ids_map['projects.survey'][UUID(sur)])
                        obj['fields']['trigger']['surveys'] = new_surveys
        self._create_objects(dump, ('account',))

    @transaction.non_atomic_requests
    def handle(self, *args, **options):
        self.create_objects('customer.json')
        self.create_objects('users.json', ('account',))
        self.create_tss()
        self.create_objects('sso_partner.json', ('customer',))
        self.create_objects('sso_users.json', ('logo', 'cw_partner', 'user'))
        self.create_configurations()

        self.create_objects('csat_companies_contacts.json', ('account', 'customer', 'source'))
        self.create_objects('csat_reactions.json', ('account','contact', 'company'))
        self.create_nps()
        self.create_objects('projects_contacts_surveys_categories.json', ('account', 'domain', 'logo', 'session'))

        self.create_projects()
        self.create_automations()



